from django.shortcuts import render, redirect
from django.contrib.auth import login
from django.contrib.auth.models import User
from django.contrib.auth import logout
from django.contrib import messages
from django.views.decorators.csrf import csrf_exempt
from django.http import JsonResponse
import csv
import os
import random
from datetime import datetime

# Fichier CSV pour stocker les scores
CSV_FILE = "scores.csv"
USER_FILE = "users.csv"
DIFFICULTY_FILE = "difficulties.csv"
DAILY_CHALLENGE_FILE = "daily_challenges.csv"
CATEGORIES = ["Au bureau (poste)", "Au bureau (espaces communs)", "Extérieur"]
LEVELS = ["Facile", "Moyen", "Difficile"]

def init_csv():
    if not os.path.exists(CSV_FILE):
        with open(CSV_FILE, 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(["user_id", "score"])

def init_user_csv():
    if not os.path.exists(USER_FILE):
        with open(USER_FILE, 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(["user_id", "username"])

#################
# Page de Login #
#################

def login_view(request):
    if request.method == "POST":
        username = request.POST.get("username")
        team = request.POST.get("team")
        user = User.objects.filter(username=username).first()
        if user is None:
            user = User.objects.create_user(username=username, password=None, last_name=team)
            user.save()
            add_user_to_csv(username, user.id)
        login(request, user)
        request.session['user_id'] = user.id
        return redirect("challenges")
    return render(request, "login.html")
        

def add_user_to_csv(username, django_user_id):
    user_id = django_user_id
    with open(USER_FILE, 'a', newline='') as file:
        writer = csv.writer(file)
        writer.writerow([user_id, username])
    return user_id

##################
# Page de Logout #
##################

def logout_view(request):
    logout(request)
    return redirect('login')

##################
# Page de Result #
##################

def ranking_view(request):
    scores = read_scores()
    users = read_users()
    teams, user_teams = read_teams()

    ranking = []
    for user_id, score in scores.items():
        if int(user_id) in users:
            ranking.append({"username": users[int(user_id)], "score": score, "user_team": user_teams[int(user_id)] if user_teams[int(user_id)] else "No Team"})

    team_scores = {}

    for entry in ranking:
        team = entry["user_team"]
        if team not in team_scores:
            team_scores[team] = 0
        team_scores[team] += entry["score"]

    team_ranking = []
    for team, score in team_scores.items():
        team_ranking.append({
            "team": team,
            "score": score
        })

    team_ranking = [team for team in team_ranking if team["team"] != "No Team"]

    ranking.sort(key=lambda x: x['score'], reverse=True)
    team_ranking.sort(key=lambda x: x["score"], reverse=True) 
    
    return render(request, "ranking.html", {"ranking": ranking, "team_ranking": team_ranking})

def read_users():
    users = {}
    for user in User.objects.all():
        users[user.id] = user.username

    return users

def read_teams():
    teams = {}
    for user in User.objects.all():
        teams[user.id] = user.last_name

    return list(set(teams)), teams

#####################
# Page de Challenge #
#####################

def challenges_view(request):
    if not request.user.is_authenticated:
        return redirect("login")
    
    user_id = request.session.get('user_id')
    
    user_challenges = read_user_daily_challenges(user_id, get_today())

    if user_challenges:
        selected_challenges = user_challenges
    else:
        selected_challenges = generate_daily_challenges(user_id)
    user_difficulty = get_user_difficulty(user_id)

    selected_challenges = read_challenges_from_ids(selected_challenges)
    return render(request, "challenges.html", {"challenges": selected_challenges, "user_difficulty": user_difficulty})

def get_user_difficulty(user_id):
    difficulty = "moyen"
    if os.path.exists(DIFFICULTY_FILE):
        with open(DIFFICULTY_FILE, "r", newline="", encoding="utf-8") as file:
            reader = csv.reader(file)
            for row in reader:
                if len(row) >= 2 and row[0] == str(user_id):
                    difficulty = row[1]
                    break
    return difficulty.capitalize()

def read_user_daily_challenges(user_id, today):
    user_challenges = []
    if os.path.exists(DAILY_CHALLENGE_FILE):
        with open(DAILY_CHALLENGE_FILE, 'r', newline='', encoding='utf-8') as file:
            reader = csv.reader(file)
            for row in reader:
                if len(row) >= 3 and row[1] == str(user_id) and row[0] == today:
                    user_challenges = row[2:]
                    break
    return user_challenges

def generate_daily_challenges(user_id):
    challenges = read_challenges()
    user_difficulty = get_user_difficulty(user_id)
    daily_challenges = []
    categories = CATEGORIES.copy()
    random.shuffle(categories)
    for category in categories:
        challenges_in_category = [c for c in challenges[user_difficulty] if c["lieu"] == category]
        if challenges_in_category:
            challenge = random.choice(challenges_in_category)
            daily_challenges.append(challenge["id"])

    save_daily_challenges_to_csv(user_id, daily_challenges)
    return daily_challenges

def save_daily_challenges_to_csv(user_id, challenge_ids):
    with open(DAILY_CHALLENGE_FILE, 'a', newline='', encoding='utf-8') as file:
        writer = csv.writer(file)
        writer.writerow([get_today(), user_id] + challenge_ids)

def update_score(user_id, points):
    scores = read_scores()
    if str(user_id) in scores:
        scores[str(user_id)] += points
    else:
        scores[str(user_id)] = points

    with open(CSV_FILE, 'w', newline='') as file:
        writer = csv.writer(file)
        writer.writerow(["user_id", "score"])
        for user_id, score in scores.items():
            writer.writerow([user_id, score])

def complete_challenge(request):
    if request.method == "POST":
        challenge_id = request.POST.get("challenge_id")
        user_id = request.session.get('user_id')
        points = get_challenge_points(challenge_id)

        if points:
            update_score(user_id, points)
            return JsonResponse({"status": "success", "points": points})

    return JsonResponse({"status": "error", "message": "Invalid request"}, status=400)

def read_challenges():
    challenges = {"Facile": [], "Moyen": [], "Difficile": []}
    if os.path.exists("challenges.csv"):
        with open("challenges.csv", 'r', newline='', encoding='utf-8') as file:
            reader = csv.reader(file)
            for row in reader:
                if len(row) >= 6:
                    challenge = {
                        "id": row[0],
                        "description": row[1],
                        "level": row[2],
                        "lieu": row[3],
                        "points": row[4],
                        "icon": row[5]
                    }
                    challenges[challenge["level"]].append(challenge)
    return challenges

def read_challenges_from_ids(challenge_ids):
    challenges = read_challenges()
    selected_challenges = []
    for level in challenges:
        for challenge in challenges[level]:
            if challenge["id"] in challenge_ids:
                selected_challenges.append(challenge)
    return selected_challenges

def get_challenge_points(challenge_id):
    challenges = read_challenges()
    for level in challenges:
        for challenge in challenges[level]:
            if challenge["id"] == challenge_id:
                return int(challenge["points"])
    return None

##################################
# Page de Mise à jour difficulty #
##################################

@csrf_exempt
def update_difficulty(request):

    if request.method == "POST":
        user_id = request.session.get('user_id')
        difficulty = request.POST.get("difficulty")
        if difficulty in ["Facile", "Moyen", "Difficile"]:
            save_user_difficulty(user_id, difficulty)
            today = get_today()
            with open(DAILY_CHALLENGE_FILE, 'r', newline='', encoding='utf-8') as file:
                reader = csv.reader(file)
                data = []
                for row in reader:
                    if len(row) >= 3 and row[1] == str(user_id) and row[0] == today:
                        continue
                    data.append(row)
            with open(DAILY_CHALLENGE_FILE, 'w', newline='', encoding='utf-8') as file:
                writer = csv.writer(file)
                for row in data:
                    writer.writerow(row)
            return JsonResponse({"status": "success"})
        else:
            return JsonResponse({"status": "error", "message": "Invalid difficulty"})
    return JsonResponse({"status": "error", "message": "Invalid request"}, status=400)

##########
# Commun #
##########

def read_scores():
    """Lire les scores depuis un fichier CSV."""
    scores = {}
    if os.path.exists(CSV_FILE):
        with open(CSV_FILE, 'r', newline='') as file:
            reader = csv.reader(file)
            next(reader)
            for row in reader:
                if len(row) >= 2:
                    scores[row[0]] = int(row[1])
    return scores

def save_user_difficulty(user_id, difficulty):
    data = []
    if os.path.exists(DIFFICULTY_FILE):
        with open(DIFFICULTY_FILE, "r", newline="", encoding="utf-8") as file:
            reader = csv.reader(file)
            for row in reader:
                if len(row) >= 2 and row[0] == str(user_id):
                    data.append([user_id, difficulty])
                else:
                    data.append(row)

    with open(DIFFICULTY_FILE, "w", newline="", encoding="utf-8") as file:
        writer = csv.writer(file)
        for row in data:
            writer.writerow(row)
        if [user_id, difficulty] not in data:
            writer.writerow([user_id, difficulty])

def get_today():
    return datetime.today().strftime('%Y-%m-%d')
